#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include "include/ThreadPool.h"
#include "include/SingleSon.h"



#define COND_LOCK(x)	 ConditionTool->Condition_Lock(x)
#define COND_UNLOCK(x)   ConditionTool->Condition_Unlock(x)
#define COND_SIGNAL(x)   ConditionTool->Condition_Signal(x)
#define COND_INIT(x)	 ConditionTool->Condition_Init(x)
#define COND_WAIT(x)	 ConditionTool->Condition_Wait(x)
#define COND_BROAD(x)    ConditionTool->Condition_Broadcast(x)
#define COND_DESTRY(x)	 ConditionTool->Condition_Destroy(x)
#define COND_TIMEWAIT(x,y) ConditionTool->Condition_Timewait(x,y)

void * Thread_Routine(void *arg)
{
	//printf("Begin Thread\r\n");
	struct timespec abstime;
	int timeout;
	ThreadPool_T  *pool=(ThreadPool_T *)arg;
	CCondition * CondTool=CSingleSon<CCondition*>::GetInstance();


	//printf("Idle Thread %d \r\n",pool->Idle);
	while(1)
	{
		timeout=0;
		CondTool->Condition_Lock(&pool->Ready);
		pool->Idle++;
		
		while(pool->First==NULL &&!pool->quit)
		{
			clock_gettime(CLOCK_REALTIME,&abstime);
			abstime.tv_sec+=2;
			int status;
			status=CondTool->Condition_Timewait(&pool->Ready,&abstime);
			if(status==ETIMEDOUT)
			{
				printf("Debug:Timeout! \n");
				timeout=1;
				break;				
			}
		}
		pool->Idle--;
		if(pool->First!=NULL)
		{
			Task_T *t=pool->First;
			pool->First=t->next;
			CondTool->Condition_Unlock(&pool->Ready);
			t->run(t->arg);
			free(t);
			CondTool->Condition_Lock(&pool->Ready);		
		}
		if(pool->quit &&pool->First==NULL)
		{
			pool->Counter--;
			if(pool->Counter==0)
			{
				CondTool->Condition_Signal(&pool->Ready);
			}
			CondTool->Condition_Unlock(&pool->Ready);
			printf("Thread Stop!\n");
			break;
		}
		if(timeout==1)
		{
			pool->Counter--;
			CondTool->Condition_Unlock(&pool->Ready);	
			printf("Thread Stop!\n");
			break;
		}
		CondTool->Condition_Unlock(&pool->Ready);

	
	}

	return NULL;

}




void CThreadPool::Thread_Init(ThreadPool_T *pool,int Threads)
{
	printf("Thread_Init\r\n");
	COND_INIT(&pool->Ready);
	pool->First=NULL;
	pool->Last=NULL;
	pool->Counter=0;
	pool->Idle=0;
	pool->Max_threads=Threads;
	pool->quit=0;	
}

void CThreadPool::Thread_Add_Task(ThreadPool_T *pool,void *(*run)(void *args),void *arg)
{
	Task_T *newtask=(Task_T *)malloc(sizeof(Task_T));
	newtask->run=run;
	newtask->arg=arg;
	newtask->next=NULL;

	COND_LOCK(&pool->Ready);
	if(pool->First==NULL)
	{
		pool->First=newtask;
		pool->Last=pool->First;
	}
	else
	{
		Task_T *Index_t=pool->Last;
		Index_t->next=newtask;
		pool->Last=Index_t->next;
	}
	
	if(pool->Idle>0)
	{
		COND_SIGNAL(&pool->Ready);	
	}
	else if(pool->Counter<pool->Max_threads)
	{
		pthread_t tid;
		pthread_create(&tid,NULL,Thread_Routine,pool);	
		printf("Thread create \r\n");
		pool->Counter++;
	}
	printf("Debug:pool->Idle =%d  pool->Counter=%d  \n",pool->Idle,pool->Counter);
	COND_UNLOCK(&pool->Ready);
}

void CThreadPool::Thread_Destroy(ThreadPool_T *Pool)
{
	if(Pool->quit)
		return;
	COND_LOCK(&Pool->Ready);
	Pool->quit=1;
	if(Pool->Counter>0)
	{
		if(Pool->Idle>0)
		{
			COND_BROAD(&Pool->Ready);
		}
		while(Pool->Counter)
		{
			COND_WAIT(&Pool->Ready);
		}
	
	}
	COND_UNLOCK(&Pool->Ready);
	COND_DESTRY(&Pool->Ready);
}
