 #include <unistd.h>
#include "include/stdafx.h"
#include "include/Condition.h"
#include "include/ThreadPool.h"
#include "include/USBModule.h"



unsigned char MDataCheckSum(unsigned char *p,int len)
{
	int i,sum=0;
	for(i=0;i<len;i++)
	{
		sum+=p[i];
	}
	return sum;
}


char Hear_chksum(char *p)
{
	//printf("Debug: in Hear_chksum \n");
	int sum=0;
	char checksum=0;
	for(int i=0;i<11;i++)
	{	
		sum+=p[i];
		//printf("Debug:p[%d]=: %d sum=%d\n",i,p[i],sum);
	}
	checksum=(sum&0xFF);
	//printf("Debug:checksum=%d sum=%d \n",checksum,sum);
	return checksum;
}


unsigned char CalcHeadCheckSum(char* buf, int size)
{
	int nSum = 0;
	for (int i=0;i<size;i++)
	{
		nSum += buf[i];
	}
	return (nSum & 0xFF);
}

unsigned char CalcHeadCheckData(char* buf, int size)
{
	int nSum = 0;
	for (int i=0;i<size;i++)
	{
		nSum += buf[i];
	}
	return nSum;
}


CUSBModule::CUSBModule():m_sensor(NULL),ConnectType(Conncet_USB),m_Connect_OK(0),m_ThreadStop(false),m_EnrollID(0),m_VerifyStop(true),TemplateSize(0)
{
	memset(m_ComOutputBuff,0,200*1024);
	memset(TemplateBuff,0,2048);
}



int CUSBModule::ConnectUSB()
{
	int ret=0;
	char *buff=NULL;
	ret=ZKFPI_Init(0x1B55,0x0121);
	//printf("Debug Device count :%d \n",ret);
	if(ret==0)
	{
		printf("ZKFPI_Init  Error \n");
		return Result_Error;
	}
	ret=ZKFPI_Open(0,&m_sensor);
	if(ret!=0)
	{
		printf("ZKFPI_Open  Error \n");
		return Result_OpenFail;
	}
	

	buff=WaitACKData(100);
	//printf("Debug:SendCommand \n");
	if(SendCommand(MD_SYS_STATUS,0,0,0,NULL,0)==0)
	{
		printf("Send STATUS fail\n");
		return Result_SendFail;
	}
#if 0
	//printf("Debug:WaitACKData \n");
	if((buff= WaitACKData(2000))==NULL)
	{
		printf("No data recv \n");
		return Result_NoReplay;
	}
	PRSHeader cmd=(PRSHeader)buff;
	if(cmd->Flag!=MD_SUCESS)
	{
		printf("Recv fail  Flag=: %d",cmd->Flag);
		return Result_NoSuccess;
	}
	printf(" Debug Recv suc cmd=0x%x  Flag=: 0x%x Param=: 0x%x  \n",cmd->Command,cmd->Flag,cmd->Param);
#endif 		
	//ZKFPI_Close(m_sensor);
	return Result_OK;
}

int CUSBModule::CloseDevice()
{
	if(ConnectType==Conncet_USB)
	{
		ZKFPI_Close(m_sensor);
		return Result_OK;
	}
	else if(ConnectType==Conncet_Serial)
	{
		return Result_OK;
	}
	else
	{
		printf("Error ConnectType! \n");
		return Result_Error;
	}
}


char *CUSBModule::WaitACKData(int timeout)
{
	char *p=NULL;
	if(ConnectType==Conncet_USB)
	{
		m_InDataLen=0;
		memset(m_ComInBuff,0,200*1024);
		int ret=ZKFPI_Read(m_sensor,m_ComInBuff,200*1024,timeout);
		if(ret>0)
		{
			p=m_ComInBuff;
			m_InDataLen=ret;
			//printf("Debug: RecvData: %s \n",p);
			return p;
		}
	}
	else if(ConnectType==Conncet_Serial)
	{
	
	}
	else 
	{
		printf("Debug: Error Conncet type \n");
		return NULL;
	}

	return p;
}

int CUSBModule::SendCommandEx(unsigned int Command,int Param,int Size,char Flag,char *databuf,int datalen)
{
	unsigned int size=0;
	int ret=0;
	PRSHeaderEx cmd=new TRSHeaderEx;
	
	//printf("Debug: m_Connect_OK: %d \n",m_Connect_OK);
	//PRSHeaderEx
	if(m_Connect_OK==0&&(Command!=MD_SYS_STATUS))
		return 0;

	
	size=sizeof(TRSHeaderEx);
	cmd->Startcode=0x70;
	cmd->Command=Command;
	cmd->Param=Param;
	cmd->Size=datalen+4;
	cmd->Flag=Flag;
	cmd->CheckSum=0;
	cmd->Endcode=0x0A;
	cmd->pData=new unsigned char[datalen];
	memcpy(cmd->pData,databuf,datalen);
	cmd->DataCheck=cmd->GetDataSum(databuf,datalen);
	printf("cmd->DataCheck= %d  \n",cmd->DataCheck);


	unsigned char *SendData=new unsigned char[cmd->Getlength()];
	cmd->Package(SendData);

	size+=datalen+4;
	if(ConnectType==Conncet_USB)
	{	
		printf("Debug:ZKFPI_Send Command:0x%x \n",Command);
		ret=ZKFPI_Send(m_sensor,(char*)SendData,cmd->Getlength());	
	}
	else if(ConnectType==Conncet_Serial)
	{
	
	}



	return ret;
}


int CUSBModule::SendCommand(unsigned int  Commond,int Param,int Size,char Flag,char *databuf,int datalen)
{
	unsigned int size=0;
	int ret=0;
	PRSHeader cmd;
	
	//printf("Debug: m_Connect_OK: %d \n",m_Connect_OK);

	if(m_Connect_OK==0&&(Commond!=MD_SYS_STATUS))
		return 0;


	cmd=(PRSHeader)m_ComOutputBuff;
	size=sizeof(TRSHeader);


	cmd->Command=Commond;
	cmd->Param=Param;
	cmd->Size=Size;
	cmd->Flag=Flag;
	cmd->Startcode=0x70;
	cmd->Endcode=0x0A;
	//cmd->pData=NULL;
	if(databuf)
	{
		memcpy(cmd+1,databuf,datalen);
		size+=datalen;
		printf("Debug:MDataCheckSum \n");
		unsigned char CheckSum=MDataCheckSum((unsigned char *)databuf,datalen);
		//*(int *)(m_ComInBuff+size)=CheckSum;
		memcpy(cmd+size,databuf,cmd->Size+4);
		size+=4;
		cmd->Size=cmd->Size+4;
	}
	

	cmd->CheckSum=Hear_chksum(m_ComOutputBuff);

	
	if(ConnectType==Conncet_USB)
	{	
		printf("Debug:ZKFPI_Send Command:0x%x \n",Commond);
		ret=ZKFPI_Send(m_sensor,(char *)cmd,size);	
	}
	else if(ConnectType==Conncet_Serial)
	{
	
	}



	return ret;

}

void *ThreadVerify(void *arg)
{
	CUSBModule *dlg=(CUSBModule *)arg;
	while(!dlg->m_VerifyStop)
		dlg->StartVerifyTemp();

	printf("Stop Verifytemp suc!\n");
}

void *ThreadFun(void *arg)
{
	printf("Start Recv Fun \n");
	CUSBModule *dlg=(CUSBModule *)arg;
	while(!dlg->m_ThreadStop)
		dlg->m_RecvData();

	printf("Stop Recv Fun Suc! \n");
}


void *ThreadCheckOperate(void *arg)
{
	ThreadAndMod_T *pThreadMode=(ThreadAndMod_T*)arg;
	CUSBModule *dlg=pThreadMode->pUSBmodules;
	Condition_T Condit={0};
	CCondition * CondTool=CSingleSon<CCondition*>::GetInstance();
	CondTool->Condition_Init(&Condit);
	while(1)
	{
		std::cout<<"----------------------------------\n Please choose:\n ------------------------\n"<<std::endl;
		std::cout<<"0.Thread_Start\n"<<"1.Thread_Stop\n"<<"2.Enroll\n"
			<<"3.Clear\n"<<"4.DeleteAllUser\n"<<"5.Connect\n"<<"6.Disconnect\n"<<"7.Verify User(1:1)\n"
			<<"8.Setting\n"<<"9.GetSetInfomation\n"<<"10.Verify Template \n"<<"11.Stop Verify Temp \n"
			<<"12.UpdataTemp\n"
			<<std::endl;
		std::cout<<"----------------------------------\n";
		int RecvChoose=0;
		std::cin>>RecvChoose;
		switch (RecvChoose)
		{
		case OP_ENROLL:
			{
				//printf("Start Enroll! \n Please input your UserID:");
				//std::cin>>dlg->m_EnrollID;
				CondTool->Condition_Lock(&Condit);
				dlg->Enroll(0);
				CondTool->Condition_Unlock(&Condit);					
			}
			break;
		case THREAD_START:
			{
				printf("Start Thread! \n");
				dlg->m_ThreadStop=false;
				CSingleSon<CThreadPool>::GetInstance().Thread_Add_Task(pThreadMode->pThreadPool,ThreadFun,dlg);
			}
			break;
		case THREAD_STOP:
			{
				printf("Stop Thread \n");
				CondTool->Condition_Lock(&Condit);
				dlg->m_ThreadStop=true;
				CondTool->Condition_Unlock(&Condit);		
			}
			break;
		case OP_CLEAR:
			{
				printf("Start Clear! \n");
				dlg->ClearModule();
			}
			break;
		case OP_DELETEUSER:
			{
				printf("Start Delete all user! \n");
				dlg->DeleteAllUser();
			}
			break;
		case OP_CONNECT:
			{			
				printf("Start to connect ! ");
				dlg->ConnectUSB();	
			}
			break;
		case OP_DISCONNECT:
			{
				printf("Start DisConnect! \n");
				dlg->CloseDevice();
			}
			break;
		case OP_VERIFY_SCAN:  //Scan if user template exists
			{
				printf("Start Verify User(1:1)! \n  please input your UserID: ");
				int UserID=0;
				std::cin>>UserID;
				dlg->StartVerify(UserID);
			}
			break;
		case OP_SETTING:
			{
				printf("Start Setting \n");

			}
			break;
		case OP_GETSETING:
			{
				printf("Get Setting infomation! \n");
				dlg->GetSetting();
			}
			break;
		case OP_SCANFTEMP:
			{
				printf("Start Verify temp \n");
				//dlg->StartVerifyTemp();
				dlg->m_VerifyStop=false;
				//printf("Debug:MaxThreadNum:%d \n",pThreadMode->pThreadPool->Max_threads);
				CSingleSon<CThreadPool>::GetInstance().Thread_Add_Task(pThreadMode->pThreadPool,ThreadVerify,dlg);
			}
			break;
		case OP_STOPVERIFYTEMP:
			{
				printf("Stop Verify ! \n");
				CondTool->Condition_Lock(&Condit);
				dlg->m_VerifyStop=true;
				CondTool->Condition_Unlock(&Condit);	
			}
			break;
		case OP_SET_TEMPELATE:
			{
				printf("Start Input your template! \n");
				dlg->SetTemplate();
			}
			break;
		default:
			{
				printf("Error command! \n");
			}
			break;
		}

	}
	
}



void CUSBModule::m_RecvData()
{
	//printf("Start Recv Fun \n");
	char *buff=NULL;
	static int i=0;
	if((buff=WaitACKData(2000))!=NULL)
	{
	   	PRSHeader cmd=(PRSHeader)buff;
		unsigned int Cmd=cmd->Command;
		//ffffff88
		Cmd=Cmd&0x000000FF;
		printf("Start recv: cmd->Command=0x%02x,cmd->Flag= 0x%02x cmd->param=0x%02x size:%d \n",Cmd,cmd->Flag,cmd->Param,cmd->Size);  
	
		if(cmd->Command==0x14)
		{
			if(cmd->Flag==0x61)
			{
				m_Connect_OK=1;
				printf("Connect OK! \n");
			}
			else if(cmd->Flag==0x63)
			{
				printf("STATUS Timeout!\n");
			}
		}
		else if(cmd->Command==0x15)
		{
			if(cmd->Flag==0x61)
			{
				printf("Enroll Succ! \n");
			}
			else if(cmd->Flag==0x62)
			{
				printf("press continue! \n");
			}
			else if(cmd->Flag==0x6c)
			{
				printf("Enroll Timeout!\n");
			}		
		}
		else if(cmd->Command==0x13)
		{
			if(cmd->Flag==0x61)
			{
					if(i==0)//Operating mode
					{
						if(cmd->Param==0x30)
						{
							printf("Verify Module! \n");
						}
						else if(cmd->Param==0x31)
						{
							printf("Read Module(pic)! \n");
						}
						else if(cmd->Param==0x32)
						{
							printf("Read Module(temp)! \n");
						}
					}
					else if(i==1)//Module ID
					{
						printf("ModuleID: 0x%0x \n",cmd->Param);
					}
					else if(i==2)//Fingerprint template
					{
						if(cmd->Param==0x30)
						{
							printf("ZK Template! \n");
						}
						else if(cmd->Param==0x32)
						{
							printf("Ansi Template! \n");
						}
						else if(cmd->Param==0x33)
						{
							printf("ISO  Template! \n");
						}				
				   }
			 i++;
			 i%=3;
			}
		
		}
		else if(Cmd==MD_ENROLL_IDCARD)
		{
			if(cmd->Flag==0x62)
				printf("Press finger continue! \n");
			else if(cmd->Flag==0x61)
			{
				printf("Enroll Succ! \n");			
				memset(TemplateBuff,0,2048);
				TemplateSize=cmd->Size-4;
				int ret= Unpackege(buff,(char *)TemplateBuff,cmd);
				if(ret==Result_Error)
				{
					printf("Data error! \n");
					memset(TemplateBuff,0,2048);
					TemplateSize=0;
				}
				else
				{
					printf("Data save ok! \n");
				}
			}
			else
				printf("Error! 0x%x\n",cmd->Flag);
		}
	}
	//m_Connect_OK=1;

}





int CUSBModule::Unpackege(char *Input,char *Output,PRSHeader cmd)
{
	int	pos=11;
	char  checksum=CalcHeadCheckSum(Input,pos);
	printf("Debug: checksum: %d, cmd->CheckSum:%d \n",checksum,cmd->CheckSum);
	if(checksum!=cmd->CheckSum)
		return Result_Error;

	pos=sizeof(TRSHeader);
	memcpy(Output,Input+pos,cmd->Size-4);
	pos+=cmd->Size-4;
	unsigned char  DataCheck=0;
	memcpy(&DataCheck,Input+pos,sizeof(unsigned char ));
	unsigned char Datacheck2=CalcHeadCheckData(Output,cmd->Size-4);
	printf("Debug: DataCheck: %d, DataCheck2: %d \n",DataCheck,Datacheck2);
	if(DataCheck!=Datacheck2)
		return Result_Error;
	return Result_OK;	
}




bool CUSBModule::ClearModule()
{
	if(SendCommand(MD_DEL_DB,0,0,0,NULL,0)==0)
	{
		printf("Send cmd fail! cmd: MD_DEL_DB \n");
		return false;
	}

	return true;

}


bool CUSBModule::DeleteAllUser()
{
	if(SendCommand(MD_DEL_ALL_USER,0,0,0,NULL,0)==0)
	{
		printf("Send cmd fail! cmd: MD_DEL_ALL_USER \n");
		return false;
	}

	return true;

}


int CUSBModule::Enroll(int Param )
{
   char *buff=NULL;
   //printf("Debug:SendCommand \n");MD_ADD_NEW
   if(SendCommand(MD_ENROLL_IDCARD,0,0,0,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }

   return Result_OK;
}


int CUSBModule::StartVerify(int param)
{
	char *buff=NULL;

   if(SendCommand(MD_VERIFY_SCAN,param,0,0x61,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }

   return Result_OK;
}


int CUSBModule::Setting()
{
	//MD_SYS_WP


}

int CUSBModule::GetSetting()
{
	//MD_SYS_RP
	char *buff=NULL;

   if(SendCommand(MD_SYS_RP,0,0,SID_MODULE_OPRATE,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }
   if(SendCommand(MD_SYS_RP,0,0,SID_MODULE_ID,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }
   if(SendCommand(MD_SYS_RP,0,0,SID_TEMP_FORMAT,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }

   return Result_OK;
}

int CUSBModule::StartVerifyTemp()
{
	char *buff=NULL;
   if(SendCommand(MD_VERIFY_IDCARD,0,0,0,NULL,0)==0)
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }
   sleep(1);
   return Result_OK;
}


int CUSBModule::SetTemplate()
{
	//MD_SET_IDCARD
	if(TemplateSize==0)
	{
		printf("No Template to pull! \n");
		return Result_Error;
	}
#if 0
	FILE *fp=NULL;
	fp= fopen("templatedata.dat","wb+");
	if(fp==NULL)
		return Result_Error;
	fwrite(TemplateBuff,1,TemplateSize,fp);
	fclose(fp);
#endif
	

   char *buff=NULL;
#if 1
   if(SendCommandEx(MD_SET_IDCARD,1,0,0,(char *)TemplateBuff,TemplateSize)==0)
#else
   if(SendCommandEx(MD_SET_IDCARD,1,0,0,NULL,TemplateSize)==0)
#endif    
   {
		printf("Send Command Fail ! \n");
		return Result_SendFail;
   }
   return Result_OK;
}