#include <unistd.h>  
#include <stdio.h>  
#include <iostream>
#include <string.h>
#include "include/ZKAPI.h"
#include "include/usb.h"



#undef  TRACE
#define TRACE			/ ## /

#define TIME_OUT		1000	//Command transmission timeout
#define MAX_COUNT		16		//Maximum number of devices


static struct usb_device  *devs[MAX_COUNT];
static unsigned char DevCnt = 0;
static int gVID, gPID;



typedef struct sensor___{
	unsigned char CurDev;
	unsigned char endpoint;
	unsigned char outEndpoint;
	bool gCapturing;
	usb_dev_handle *device;
}sensor_t;

int verbose = 0;

void print_endpoint(struct usb_endpoint_descriptor *endpoint)
{
  printf("      bEndpointAddress: %02xh\n", endpoint->bEndpointAddress);
  printf("      bmAttributes:     %02xh\n", endpoint->bmAttributes);
  printf("      wMaxPacketSize:   %d\n", endpoint->wMaxPacketSize);
  printf("      bInterval:        %d\n", endpoint->bInterval);
  printf("      bRefresh:         %d\n", endpoint->bRefresh);
  printf("      bSynchAddress:    %d\n", endpoint->bSynchAddress);
}

void print_altsetting(struct usb_interface_descriptor *interface)
{
  int i;

  printf("    bInterfaceNumber:   %d\n", interface->bInterfaceNumber);
  printf("    bAlternateSetting:  %d\n", interface->bAlternateSetting);
  printf("    bNumEndpoints:      %d\n", interface->bNumEndpoints);
  printf("    bInterfaceClass:    %d\n", interface->bInterfaceClass);
  printf("    bInterfaceSubClass: %d\n", interface->bInterfaceSubClass);
  printf("    bInterfaceProtocol: %d\n", interface->bInterfaceProtocol);
  printf("    iInterface:         %d\n", interface->iInterface);

  for (i = 0; i < interface->bNumEndpoints; i++)
    print_endpoint(&interface->endpoint[i]);
}

void print_interface(struct usb_interface *interface)
{
  int i;

  for (i = 0; i < interface->num_altsetting; i++)
    print_altsetting(&interface->altsetting[i]);
}

void print_configuration(struct usb_config_descriptor *config)
{
  int i;

  printf("  wTotalLength:         %d\n", config->wTotalLength);
  printf("  bNumInterfaces:       %d\n", config->bNumInterfaces);
  printf("  bConfigurationValue:  %d\n", config->bConfigurationValue);
  printf("  iConfiguration:       %d\n", config->iConfiguration);
  printf("  bmAttributes:         %02xh\n", config->bmAttributes);
  printf("  MaxPower:             %d\n", config->MaxPower);

  for (i = 0; i < config->bNumInterfaces; i++)
    print_interface(&config->interface[i]);
}

int print_device(struct usb_device *dev, int level)
{
  usb_dev_handle *udev;
  char description[256];
  char string[256];
  int ret, i;

  udev = usb_open(dev);
  if (udev) {
    if (dev->descriptor.iManufacturer) {
      ret = usb_get_string_simple(udev, dev->descriptor.iManufacturer, string, sizeof(string));
      if (ret > 0)
        snprintf(description, sizeof(description), "%s - ", string);
      else
        snprintf(description, sizeof(description), "%04X - ",
                 dev->descriptor.idVendor);
    } else
      snprintf(description, sizeof(description), "%04X - ",
               dev->descriptor.idVendor);

    if (dev->descriptor.iProduct) {
      ret = usb_get_string_simple(udev, dev->descriptor.iProduct, string, sizeof(string));
      if (ret > 0)
        snprintf(description + strlen(description), sizeof(description) -
                 strlen(description), "%s", string);
      else
        snprintf(description + strlen(description), sizeof(description) -
                 strlen(description), "%04X", dev->descriptor.idProduct);
    } else
      snprintf(description + strlen(description), sizeof(description) -
               strlen(description), "%04X", dev->descriptor.idProduct);

  } else
    snprintf(description, sizeof(description), "%04X - %04X",
             dev->descriptor.idVendor, dev->descriptor.idProduct);

  printf("%.*sDev #%d: %s\n", level * 2, "                    ", dev->devnum,
         description);

  if (udev && verbose) {
    if (dev->descriptor.iSerialNumber) {
      ret = usb_get_string_simple(udev, dev->descriptor.iSerialNumber, string, sizeof(string));
      if (ret > 0)
        printf("%.*s  - Serial Number: %s\n", level * 2,
               "                    ", string);
    }
  }

  if (udev)
    usb_close(udev);

  if (verbose) {
    if (!dev->config) {
      printf("  Couldn't retrieve descriptors\n");
      return 0;
    }

    for (i = 0; i < dev->descriptor.bNumConfigurations; i++)
      print_configuration(&dev->config[i]);
  } else {
    for (i = 0; i < dev->num_children; i++)
      print_device(dev->children[i], level + 1);
  }

  return 0;
}





int ZKFPI_Init(unsigned short VID, unsigned short PID)
{
	struct usb_bus *bus;
	struct usb_device *dev;

	usb_init();

	if(usb_find_busses()<0)
		return 0;
	if(usb_find_devices()<0)
		return 0;

	DevCnt=0;

	for(bus=usb_get_busses();bus;bus=bus->next)
	{
		for(dev=bus->devices;dev;dev=dev->next)
		{
			if(DevCnt>MAX_COUNT)
				break;
			if(dev->descriptor.idVendor==VID &&
				dev->descriptor.idProduct==PID
				)
			{
				devs[DevCnt++]=dev;
			}
		}
	}


	//printf("Debug: ZKFPI_Init OK ! Found %d devices !\n",DevCnt);
	return DevCnt;
}


int ZKFPI_Open(int index, void **sensor)
{
	sensor_t s;
	int ret=-ENODEV;
	struct usb_endpoint_descriptor *pEndpoint=NULL;

	memset(&s,0,sizeof(s));
	if(index>=DevCnt)
		return ret;

	if(devs[index]==NULL)
		return ret;

	usb_dev_handle *device=NULL;
	s.CurDev=index;


	//print_device(devs[s.CurDev], 0);
	device = usb_open(devs[s.CurDev]);
	if(device != NULL)	{
		if((ret=usb_set_configuration(device,1))<0)
		{
			usb_close(device);
			return ret;
		}
		if((ret=usb_claim_interface(device,0))<0)
		{
			usb_close(device);
			return ret;
		}
		pEndpoint=devs[s.CurDev]->config->interface->altsetting->endpoint;
		for(int i=0;i<devs[s.CurDev]->config->interface->altsetting->bNumEndpoints;i++)
		{
			if(pEndpoint->bmAttributes==USB_ENDPOINT_TYPE_BULK)
			{
				if(1==(pEndpoint->bEndpointAddress>>7))
				{
					s.endpoint=pEndpoint->bEndpointAddress;
				}
				else
				{
					s.outEndpoint=pEndpoint->bEndpointAddress;
				}
			
			}
			pEndpoint++;		
		}
		s.device=device;
		*sensor=(sensor_t *)malloc(sizeof(sensor_t));
		memcpy(*sensor,&s,sizeof(s));
		//printf("Debug: ZKFPI_Open OK !\n");
		return 0;
	}
	printf("Debug: ZKFPI_Open Fail !\n");
	
	return ret;
}


int ZKFPI_Send(void *sensor, char *buffer, int len)
{
	int ret=0;
	sensor_t *s=(sensor_t *)sensor;
	if(s==NULL) return -1;
	if(s->device==NULL)return -1;

	if(s->device!=NULL)
	{
		if((ret=usb_control_msg(s->device,USB_TYPE_VENDOR|USB_RECIP_DEVICE|USB_ENDPOINT_OUT,0xFA,0,0,buffer,len,TIME_OUT))<0)
		{
			printf("Debug : ZKFPI_SetGPIO Fail ! \n");
			return -1;
		}
	
	}
	return ret;
}

int ZKFPI_Close(void *sensor)
{
	sensor_t *s=(sensor_t *)sensor;
	if(s==NULL)return -1;

	if(s->device!=	NULL)
	{
		if(0==usb_release_interface(s->device,0))
		{
			usb_close(s->device);			
		}
		s->device=NULL;

		printf("Debug:ZKFPI_Close OK! \n");
	}
	free(s);
	return 0;
}


int ZKFPI_Read(void *sensor,char *buffer,int len,int timeout)
{
	int ret=0;
	sensor_t *s=(sensor_t *)sensor;

	if(s==NULL)return -1;
	if(s->device==	NULL)return -1;
	if((ret=usb_bulk_read(s->device,s->endpoint,(char *)buffer,len,timeout))<=0)
	{
		return ret;
	}
	return ret;
}

int ZKFPI_Write(void *sensor,char *buffer,int len,int timeout)
{
	int ret=0;
	sensor_t *s=(sensor_t *)sensor;
	if(s==NULL)return -1;
	if(s->device==	NULL)return -1;

	if((ret=usb_bulk_write(s->device,s->endpoint,(char *)buffer,len,timeout))<=0)
	{
		char *str=usb_strerror();
		return -1;
	}
	return ret;
}